@@ -14,6 +14,7 @@ from scipy.sparse import linalg as SPla
 import scipy.optimize as op
 import numpy.linalg as la
 from pysal import lag_spatial
+from sputils import *
 import copy
 
 
@@ -535,7 +536,7 @@ def inverse_prod(w, data, scalar, post_multiply=False, inv_method="power_exp", t
         else:
             inv_prod = spdot(matrix, data)
     else:
-        raise Exception("Invalid method selected for inversion.")
+        raise Exception, "Invalid method selected for inversion."
     return inv_prod
 
 
@@ -574,7 +575,7 @@ def power_expansion(w, data, scalar, post_multiply=False, threshold=0.0000000001
         test_old = test
         test = la.norm(increment)
         if test > test_old:
-            raise Exception("power expansion will not converge, check model specification and that weight are less than 1")
+            raise Exception, "power expansion will not converge, check model specification and that weight are less than 1"
         count += 1
     return running_total
 
@@ -595,7 +596,7 @@ def set_endog(y, x, w, yend, q, w_lags, lag_q):
         q = get_lags(w, x, w_lags)
         yend = yl
     else:
-        raise Exception("invalid value passed to yend")
+        raise Exception, "invalid value passed to yend"
     return yend, q
 
     lag = lag_spatial(w, x)
@@ -628,7 +629,7 @@ def set_endog_sparse(y, x, w, yend, q, w_lags, lag_q):
             q = sphstack(q, w * q)
         yend = yl
     else:
-        raise Exception("invalid value passed to yend")
+        raise Exception, "invalid value passed to yend"
     return yend, q
 
 
@@ -657,213 +658,6 @@ def sp_att(w, y, predy, w_y, rho):
     # see Note 1.
     return predy_sp, resid_sp, warn
 
-
-def spdot(a, b, array_out=True):
-    """
-    Matrix multiplication function to deal with sparse and dense objects
-
-    Parameters
-    ----------
-
-    a           : array
-                  first multiplication factor. Can either be sparse or dense.
-    b           : array
-                  second multiplication factor. Can either be sparse or dense.
-    array_out   : boolean
-                  If True (default) the output object is always a np.array
-
-    Returns
-    -------
-
-    ab : array
-         product of a times b. Sparse if a and b are sparse. Dense otherwise.
-    """
-    if type(a).__name__ == 'ndarray' and type(b).__name__ == 'ndarray':
-        ab = np.dot(a, b)
-    elif type(a).__name__ == 'csr_matrix' or type(b).__name__ == 'csr_matrix' \
-            or type(a).__name__ == 'csc_matrix' or type(b).__name__ == 'csc_matrix':
-        ab = a * b
-        if array_out:
-            if type(ab).__name__ == 'csc_matrix' or type(ab).__name__ == 'csr_matrix':
-                ab = ab.toarray()
-    else:
-        raise Exception("Invalid format for 'spdot' argument: %s and %s" % (
-            type(a).__name__, type(b).__name__))
-    return ab
-
-
-def spmultiply(a, b, array_out=True):
-    """
-    Element-wise multiplication function to deal with sparse and dense
-    objects. Both objects must be of the same type.
-
-    Parameters
-    ----------
-
-    a           : array
-                  first multiplication factor. Can either be sparse or dense.
-    b           : array
-                  second multiplication factor. Can either be sparse or dense.
-                  integer.
-    array_out   : boolean
-                  If True (default) the output object is always a np.array
-
-    Returns
-    -------
-
-    ab : array
-         elementwise multiplied object. Sparse if a is sparse. Dense otherwise.
-    """
-    if type(a).__name__ == 'ndarray' and type(b).__name__ == 'ndarray':
-        ab = a * b
-    elif (type(a).__name__ == 'csr_matrix' or type(a).__name__ == 'csc_matrix') \
-            and (type(b).__name__ == 'csr_matrix' or type(b).__name__ == 'csc_matrix'):
-        ab = a.multiply(b)
-        if array_out:
-            if type(ab).__name__ == 'csc_matrix' or type(ab).__name__ == 'csr_matrix':
-                ab = ab.toarray()
-    else:
-        raise Exception("Invalid format for 'spmultiply' argument: %s and %s" % (
-            type(a).__name__, type(b).__name__))
-    return ab
-
-
-def sphstack(a, b, array_out=False):
-    """
-    Horizontal stacking of vectors (or matrices) to deal with sparse and dense objects
-
-    Parameters
-    ----------
-
-    a           : array or sparse matrix
-                  First object.
-    b           : array or sparse matrix
-                  Object to be stacked next to a
-    array_out   : boolean
-                  If True the output object is a np.array; if False (default)
-                  the output object is an np.array if both inputs are
-                  arrays or CSR matrix if at least one input is a CSR matrix
-
-    Returns
-    -------
-
-    ab          : array or sparse matrix
-                  Horizontally stacked objects
-    """
-    if type(a).__name__ == 'ndarray' and type(b).__name__ == 'ndarray':
-        ab = np.hstack((a, b))
-    elif type(a).__name__ == 'csr_matrix' or type(b).__name__ == 'csr_matrix':
-        ab = SP.hstack((a, b), format='csr')
-        if array_out:
-            if type(ab).__name__ == 'csr_matrix':
-                ab = ab.toarray()
-    else:
-        raise Exception("Invalid format for 'sphstack' argument: %s and %s" % (
-            type(a).__name__, type(b).__name__))
-    return ab
-
-
-def spbroadcast(a, b, array_out=False):
-    """
-    Element-wise multiplication of a matrix and vector to deal with sparse 
-    and dense objects
-
-    Parameters
-    ----------
-
-    a           : array or sparse matrix
-                  Object with one or more columns.
-    b           : array
-                  Object with only one column
-    array_out   : boolean
-                  If True the output object is a np.array; if False (default)
-                  the output object is an np.array if both inputs are
-                  arrays or CSR matrix if at least one input is a CSR matrix
-
-    Returns
-    -------
-
-    ab          : array or sparse matrix
-                  Element-wise multiplication of a and b
-    """
-    if type(a).__name__ == 'ndarray' and type(b).__name__ == 'ndarray':
-        ab = a * b
-    elif type(a).__name__ == 'csr_matrix':
-        b_mod = SP.lil_matrix((b.shape[0], b.shape[0]))
-        b_mod.setdiag(b)
-        ab = (a.T * b_mod).T
-        if array_out:
-            if type(ab).__name__ == 'csr_matrix':
-                ab = ab.toarray()
-    else:
-        raise Exception("Invalid format for 'spbroadcast' argument: %s and %s" % (
-            type(a).__name__, type(b).__name__))
-    return ab
-
-
-def spmin(a):
-    """
-    Minimum value in a matrix or vector to deal with sparse and dense objects
-
-    Parameters
-    ----------
-
-    a           : array or sparse matrix
-                  Object with one or more columns.
-
-    Returns
-    -------
-
-    min a       : int or float
-                  minimum value in a
-    """
-
-    if type(a).__name__ == 'ndarray':
-        return a.min()
-    elif type(a).__name__ == 'csr_matrix' or type(a).__name__ == 'csc_matrix':
-        try:
-            return min(a.data)
-        except:
-            if np.sum(a.data) == 0:
-                return 0
-            else:
-                raise Exception("Error: could not evaluate the minimum value.")
-    else:
-        raise Exception("Invalid format for 'spmultiply' argument: %s and %s" % (
-            type(a).__name__, type(b).__name__))
-
-
-def spmax(a):
-    """
-    Maximum value in a matrix or vector to deal with sparse and dense objects
-
-    Parameters
-    ----------
-
-    a           : array or sparse matrix
-                  Object with one or more columns.
-
-    Returns
-    -------
-
-    max a       : int or float
-                  maximum value in a
-    """
-    if type(a).__name__ == 'ndarray':
-        return a.max()
-    elif type(a).__name__ == 'csr_matrix' or type(a).__name__ == 'csc_matrix':
-        try:
-            return max(a.data)
-        except:
-            if np.sum(a.data) == 0:
-                return 0
-            else:
-                raise Exception("Error: could not evaluate the maximum value.")
-    else:
-        raise Exception("Invalid format for 'spmultiply' argument: %s and %s" % (
-            type(a).__name__, type(b).__name__))
-
-
 def set_warn(reg, warn):
     ''' Groups warning messages for printout. '''
     if warn:
@@ -874,26 +668,6 @@ def set_warn(reg, warn):
     else:
         pass
 
-def splogdet(spmat):
-    ''' Large dimension log determinant computation. '''
-    #symmetric = np.allclose(spmat.T, spmat) #could branch for cholmod
-    symmetric = False
-    if symmetric:
-        #CHOLMOD could be used like:
-        #from scikits.sparse import cholmod as CHOLMOD
-        #det = np.sum(np.log(np.abs(CHOLMOD.cholesky(spmat).D())))
-        pass #leave in for scaffolding.
-    else:
-        if isinstance(spmat, SP.csc_matrix) or isinstance(spmat, SP.csr_matrix):
-            LU = SPla.splu(spmat)
-            det = np.sum(np.log(np.abs(LU.U.diagonal())))
-        elif SP.isspmatrix(spmat):
-            return spdet(spmat.tocsc())
-        else:
-            det = la.slogdet(spmat)
-    return det
-
-
 def RegressionProps_basic(reg, betas=None, predy=None, u=None, sig2=None, sig2n_k=None, vm=None):
     ''' Set props based on arguments passed. '''
     if betas is not None:
